{**
	CESYS Software License - Version 1.0 - January 1st, 2016

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The software must only be used to operate hardware manufactured by 
	CESYS GmbH, Herzogenaurach.

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
  
	CeboMsr - Delphi example - Single I/O.

	The I/O's of digital port #0 are read and mirrored to digital port #1.
	(IO-0 - IO-7 are mirrored to IO-8 - IO-15).
	The voltage at AI-0 is mirrored to AO-1 and AI-1 to AO-0 too.

	Does only work on CeboLC devices.

	@author Daniel Schroll
}

program SingleIoExample;
	uses
		SysUtils,
		CeboMsr in 'cebomsr\CeboMsr.pas';

	procedure SingleIoTest(Device: TDevice);
		var
			Dp0, Dp1: TDigitalPort;
			Value: longint;
			VoltageValue: single;
	begin

		//// Read digital port #0 and write result to digital port #1 (mirror pins).

		// At first, we need references to both ports.
		Dp0 := Device.DigitalPorts[0];
		Dp1 := Device.DigitalPorts[1];

		// Configure port first, all bits of digital port #0 as input (default)
		// and all bits of digital port #1 as output.
		Dp1.OutputEnableMask := $ff;

		// Read from #0 ...
		Value := Dp0.Value;

		// Write the value to #1 ...
		Dp1.Value := Value;

		//// Now some analog I/O
		
		// do it without any additional local references.
		// Read single ended #0 input voltage ...
		VoltageValue := Device.SingleEndedInputs[0].Value;

		// Write value to analog output #1
		Device.AnalogOutputs[1].Value := VoltageValue;

		// .. or in one line: single ended #1 to analog output #0 ...
		Device.AnalogOutputs[0].Value := Device.SingleEndedInputs[1].Value;

		// Reset to power up defaults.
		Device.ResetDevice();
	end;

var
	Devices: TDeviceList;
	Device: TDevice;

begin
	try
		// Search for devices
		Devices := CeboMsr.TLibraryInterface.Enumerate(CeboMsr.DeviceType.AllDevices);

		// If at least one device has been found, use the first one
		if (length(Devices) > 0) then begin
			Device := Devices[0];

			// Open device, nothing can be done without doing this
			Device.Open();

			// *** DIGITAL PORTS AND ANALOG OUTPUTS ARE MODIFIED DURING THIS ***
			// *** SECTION, SO DO NOT CONNECT ANY PERIPHERALS WITHOUT STUDYING ***
			// *** THE ACTIONS DONE HERE, OTHERWISE YOUR PERIPHERALS MAY BE DAMAGED !!! ***
			SingleIoTest(Device);

			// Finalize device usage, this free's up the device, so it can be used
			// again, including other applications.
			Device.Close();
		end else begin
			writeln('No Devices found!');
		end;
		readln;
	except
		on E: EArgumentException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EIndexOutOfRangeException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EInvalidOperationException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EIoException do begin
			writeln(E.Message);
			readln;
		end;
	end;
end.
